import click
import getpass
import os
import sqlite3

from bootstrap_budget import Users
from importlib.resources import files


def create_schema() -> None:
    db_schema_script: str = files('bootstrap_budget').joinpath('db/sqlite/create_sqlite_schema.sql').read_text()
    db_connection: sqlite3.Connection = sqlite3.connect('bootstrap_budget.db')
    sql_cursor: sqlite3.Cursor = db_connection.cursor()

    # Iterate through each SQL statement in the file
    for schema_definition in db_schema_script.split('--'):
        response = sql_cursor.execute(schema_definition)

    db_connection.close()


@click.command('bootstrap-init')
@click.option('-set-admin-passwd', is_flag=True, help='Set admin password.')
@click.option('-refresh-schema', is_flag=True, help='Refresh the database schema (drop and replace).')
def bootstrap_init(set_admin_passwd: bool, refresh_schema: bool) -> None:
    if os.path.exists('bootstrap_budget.db'):
        if refresh_schema:
            create_schema()
            click.echo('Bootstrap-Budget schema has been refreshed.')
        else:
            click.echo('Bootstrap-Budget schema already exists.')
    else:
        create_schema()
        click.echo('Bootstrap-Budget schema has been created.')

    passwd: bytes

    if set_admin_passwd:
        passwd = bytes(getpass.getpass('Enter admin password:'), 'utf-8')
    else:
        passwd = bytes('admin', 'utf-8')

    admin_user = Users(username='admin', is_admin=True)
    admin_user.create(user_password=passwd)
    click.echo('Admin user created.')


if __name__ == '__main__':
    pass



import bcrypt
import datetime
import sqlite3

from importlib.resources import files


def hash_password(password: bytes) -> tuple:
    salt = bcrypt.gensalt()
    return bcrypt.hashpw(password, salt), salt


def validate_password(password: bytes, password_hash: bytes):
    return bcrypt.checkpw(password, password_hash)


class Users:
    # USERS table fields
    last_name: str
    first_name: str
    middle_name: str
    username: str
    address_line_1: str
    address_line_2: str
    city: str
    state: str
    zipcode: str
    email: str
    phone_number: str
    hash: str
    salt: str
    is_admin: bool
    created_dt_tm: datetime
    updated_dt_tm: datetime
    is_active: bool

    # Constants
    EMPTY_STRING: str = ''

    def __init__(self, username: str, **fields) -> None:
        self.last_name = fields.get('last_name', self.EMPTY_STRING)
        self.first_name = fields.get('first_name', self.EMPTY_STRING)
        self.middle_name = fields.get('middle_name', self.EMPTY_STRING)
        self.username = username
        self.address_line_1 = fields.get('address_line_1', self.EMPTY_STRING)
        self.address_line_2 = fields.get('address_line_2', self.EMPTY_STRING)
        self.city = fields.get('city', self.EMPTY_STRING)
        self.state = fields.get('state', self.EMPTY_STRING)
        self.zipcode = fields.get('zipcode', self.EMPTY_STRING)
        self.email = fields.get('email', self.EMPTY_STRING)
        self.phone_number = fields.get('phone_number', self.EMPTY_STRING)
        self.is_admin = fields.get('is_admin', False)
        self.is_active = fields.get('is_active', True)

    def create(self, user_password: bytes) -> None:
        insert_user: str = files('bootstrap_budget').joinpath('db/sqlite/create_user.sql').read_text()
        db_connection: sqlite3.Connection = sqlite3.connect(f'bootstrap_budget.db')
        sql_cursor: sqlite3.Cursor = db_connection.cursor()

        # Generate password hash and salt
        hashed_password, password_salt = hash_password(user_password)

        # Capture current datetime for creation and update timestamps
        current_datetime = datetime.datetime.now()
        current_datetime_iso = current_datetime.isoformat()

        try:
            response = sql_cursor.execute(insert_user, [
                self.last_name,
                self.first_name,
                self.middle_name,
                self.username,
                self.address_line_1,
                self.address_line_2,
                self.city,
                self.state,
                self.zipcode,
                self.email,
                self.phone_number,
                hashed_password,  # hash
                password_salt,  # salt
                self.is_admin,
                current_datetime_iso,  # created_dt_tm
                current_datetime_iso,  # updated_dt_tm
                self.is_active
            ])

            db_connection.commit()
        except Exception as e:
            print(e)

        db_connection.close()



from flask import Flask, render_template
from logging.config import dictConfig

# Import bootstrap-budget modules/classes/functions
from .users import Users


dictConfig({
    'version': 1,
    'formatters': {
        'default': {
            'format': '[%(asctime)s] %(levelname)s in %(module)s: %(message)s',
        }
    },
    'handlers': {
        'wsgi': {
            'class': 'logging.StreamHandler',
            'stream': 'ext://flask.logging.wsgi_errors_stream',
            'formatter': 'default'
        }
    },
    'root': {
        'level': 'INFO',
        'handlers': ['wsgi']
    }
})


def start(test_config=None):
    # Create and configure the app
    app = Flask(__name__)

    @app.route("/")
    def index():
        return render_template('index.html')

    return app



from setuptools import setup


# Use README file as long description
with open("README.md") as f:
    long_description = f.read()


setup(
    name='bootstrap-budget',
    version='0.0.2',
    author='forgineer',
    description='A simple financial application to help you pull your budget up by its bootstraps.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/forgineer/bootstrap-budget',
    license='MIT License',
    packages=['bootstrap_budget'],
    include_package_data=True,
    python_requires='>=3.7',
    install_requires=[
        'bcrypt'
        'flask'
    ],
    extras_require={  # pip install -e .[pypi_deployment]
        'docs': [
            'mkdocs'
        ],
        'deployment': [
            'build',
            'twine'
        ]
    },
    entry_points={
        'console_scripts': [
            'bootstrap-init = bootstrap_budget.cli:bootstrap_init'
        ]
    },
    keywords='personal budget,budgeting,web app',
    # https://pypi.org/classifiers/
    classifiers=[
        'Development Status :: 1 - Planning',
        'Framework :: Flask',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Office/Business :: Financial'
    ]
)

"""
Build and deploy steps:
    - python -m build
    - twine check dist/*
    - twine upload -r testpypi dist/*
    - twine upload dist/*
"""
